@@ -1,8 +1,9 @@
 ;; camlp5 ./pa_lispr.cmo pa_extend.cmo q_MLast.cmo pr_dump.cmo
-;; $Id: pa_lisp.ml,v 6.9 2012-01-09 14:22:20 deraugla Exp $
-;; Copyright (c) INRIA 2007-2012
+;; pa_lisp.ml,v
+;; Copyright (c) INRIA 2007-2014
 
 (open Pcaml)
+(open Versdep)
 
 (type (choice 'a 'b) (sum (Left 'a) (Right 'b)))
 
@@ -10,13 +11,13 @@
 
 (module Buff
  (struct
-  (value buff (ref (String.create 80)))
+  (value buff (ref (string_create 80)))
   (value store (lambda (len x)
                  (if (>= len (String.length buff.val))
                      (:= buff.val
                          (^ buff.val
-                            (String.create (String.length buff.val)))))
-                 (:= ([] buff.val len) x)
+                            (string_create (String.length buff.val)))))
+                 (string_set buff.val len x)
                  (succ len)))
   (value get (lambda len (String.sub buff.val 0 len)))))
 
