@@ -1,9 +1,10 @@
-(* camlp5r pa_macro.cmo *)
-(* $Id: pa_o.ml,v 6.50 2013-07-02 16:12:43 deraugla Exp $ *)
-(* Copyright (c) INRIA 2007-2012 *)
+(* camlp5r *)
+(* pa_o.ml,v *)
+(* Copyright (c) INRIA 2007-2014 *)
 
 #load "pa_extend.cmo";
 #load "q_MLast.cmo";
+#load "pa_macro.cmo";
 
 open Pcaml;
 
@@ -253,7 +254,7 @@ value test_label_eq =
        [ Some (("UIDENT", _) | ("LIDENT", _) | ("", ".")) ->
            test (lev + 1) strm
        | Some ("ANTIQUOT_LOC", _) -> ()
-       | Some ("", "=") -> ()
+       | Some ("", "=" | ";" | "}") -> ()
        | _ -> raise Stream.Failure ])
 ;
 
@@ -356,6 +357,13 @@ value concat_comm loc e =
   Reloc.expr floc 0 e
 ;
 
+value expr_of_patt p =
+  let loc = MLast.loc_of_patt p in
+  match p with
+  [ <:patt< $lid:x$ >> -> <:expr< $lid:x$ >>
+  | _ -> Ploc.raise loc (Stream.Error "identifier expected") ]
+;
+
 EXTEND
   GLOBAL: sig_item str_item ctyp patt expr module_type module_expr
     signature structure class_type class_expr class_sig_item class_str_item
@@ -525,6 +533,8 @@ EXTEND
       | "let"; "module"; m = V UIDENT; mb = mod_fun_binding; "in";
         e = expr LEVEL "top" ->
           <:expr< let module $_uid:m$ = $mb$ in $e$ >>
+      | "let"; "open"; m = V UIDENT; "in"; e = expr LEVEL "top" ->
+          <:expr< $_uid:m$ . ($e$) >>
       | "function"; OPT "|"; l = V (LIST1 match_case SEP "|") ->
           <:expr< fun [ $_list:l$ ] >>
       | "fun"; p = patt LEVEL "simple"; (eo, e) = fun_def ->
@@ -697,7 +707,8 @@ EXTEND
       | le = lbl_expr -> [le] ] ]
   ;
   lbl_expr:
-    [ [ i = patt_label_ident; "="; e = expr LEVEL "expr1" -> (i, e) ] ]
+    [ [ i = patt_label_ident; "="; e = expr LEVEL "expr1" -> (i, e)
+      | i = patt_label_ident -> (i, expr_of_patt i) ] ]
   ;
   expr1_semi_list:
     [ [ el = LIST1 (expr LEVEL "expr1") SEP ";" OPT_SEP -> el ] ]
@@ -817,6 +828,7 @@ EXTEND
   ;
   lbl_patt:
     [ [ i = patt_label_ident; "="; p = patt -> (i, p)
+      | i = patt_label_ident -> (i, i)
       | "_" -> (<:patt< _ >>, <:patt< _ >>) ] ]
   ;
   patt_label_ident:
