@@ -1,12 +1,13 @@
 (* camlp5r *)
-(* $Id: pa_pprintf.ml,v 6.4 2012-01-09 14:22:21 deraugla Exp $ *)
-(* Copyright (c) INRIA 2007-2012 *)
+(* pa_pprintf.ml,v *)
+(* Copyright (c) INRIA 2007-2014 *)
 
 #load "pa_extend.cmo";
 #load "pa_fstream.cmo";
 #load "q_MLast.cmo";
 
 open Pcaml;
+open Versdep;
 
 (** Types and Functions for the [pprintf] statement *)
 
@@ -33,10 +34,10 @@ type tree =
 ;
 
 value implode l =
-  let s = String.create (List.length l) in
+  let s = string_create (List.length l) in
   loop 0 l where rec loop i =
     fun
-    [ [c :: l] -> do { String.set s i c; loop (i + 1) l }
+    [ [c :: l] -> do { string_set s i c; loop (i + 1) l }
     | [] -> s ]
 ;
 
