@@ -1,6 +1,6 @@
 (* camlp5r *)
-(* $Id: pa_pragma.ml,v 6.6 2012-01-09 14:22:21 deraugla Exp $ *)
-(* Copyright (c) INRIA 2007-2012 *)
+(* pa_pragma.ml,v *)
+(* Copyright (c) INRIA 2007-2014 *)
 
 #load "pa_macro.cmo";
 #load "q_MLast.cmo";
@@ -999,7 +999,7 @@ value val_tab = do {
      ("String.set",
       fun loc ->
         {ctyp = <:ctyp< string -> int -> char -> unit >>;
-         expr = Obj.repr String.set;
+         expr = Obj.repr string_set;
          patt = no_patt loc});
      ("String.sub",
       fun loc ->
