@@ -1,9 +1,10 @@
 ; camlp5 ./pa_schemer.cmo pa_extend.cmo q_MLast.cmo pr_dump.cmo
-; $Id: pa_scheme.ml,v 6.9 2012-01-09 14:22:21 deraugla Exp $
-; Copyright (c) INRIA 2007-2012
+; pa_scheme.ml,v
+; Copyright (c) INRIA 2007-2014
 
 (open Pcaml)
 (open Exparser)
+(open Versdep)
 
 (type (choice 'a 'b)
  (sum
@@ -14,12 +15,12 @@
 
 (module Buff
  (struct
-  (define buff (ref (String.create 80)))
+  (define buff (ref (string_create 80)))
   (define (store len x)
    (begin
     (if (>= len (String.length buff.val))
-     (:= buff.val (^ buff.val (String.create (String.length buff.val)))))
-    (:= buff.val.[len] x)
+     (:= buff.val (^ buff.val (string_create (String.length buff.val)))))
+    (string_set buff.val len x)
     (succ len)))
   (define (mstore len s)
    (letrec
@@ -488,7 +489,7 @@
      <:sig_item< module $flag:rf$ $_list:lmb$ >>))
    ((Sexpr loc [(Slid _ "moduletype") se1 se2])
     (let* ((s (anti_uid_or_error se1)) (mt (module_type_se se2)))
-     <:sig_item< module type $_uid:s$ = $mt$ >>))
+     <:sig_item< module type $_:s$ = $mt$ >>))
    ((Sexpr loc [(Slid _ "open") se])
     (let ((s (anti_longident_se se))) <:sig_item< open $_:s$ >>))
    ((Sexpr loc [(Slid _ "type") . sel])
@@ -543,7 +544,7 @@
      <:str_item< module $flag:rf$ $_list:lmb$ >>))
    ((Sexpr loc [(Slid _ "moduletype") se1 se2])
     (let* ((s (anti_uid_or_error se1)) (mt (module_type_se se2)))
-     <:str_item< module type $_uid:s$ = $mt$ >>))
+     <:str_item< module type $_:s$ = $mt$ >>))
    ((Sexpr loc [(Slid _ "open") se])
     (let ((s (anti_longident_se se))) <:str_item< open $_:s$ >>))
    ((Sexpr loc [(Slid _ "type") . sel])
