@@ -1,6 +1,6 @@
 (* camlp5r *)
-(* $Id: pa_sml.ml,v 6.5 2011-03-15 13:49:10 deraugla Exp $ *)
-(* Copyright (c) INRIA 2007-2011 *)
+(* pa_sml.ml,v *)
+(* Copyright (c) INRIA 2007-2014 *)
 
 #load "pa_extend.cmo";
 #load "q_MLast.cmo";
@@ -808,7 +808,7 @@ EXTEND
   ;
   sigb:
     [ [ x1 = ident; "="; x2 = module_type ->
-          <:str_item< module type $uid:x1$ = $x2$ >> ] ]
+          <:str_item< module type $x1$ = $x2$ >> ] ]
   ;
   fsig:
     [ [ ":"; x1 = ident -> not_impl loc "fsig 1"
