@@ -1,6 +1,6 @@
 (* camlp5r *)
-(* $Id: pr_o.ml,v 6.63 2013-07-02 16:12:43 deraugla Exp $ *)
-(* Copyright (c) INRIA 2007-2012 *)
+(* pr_o.ml,v *)
+(* Copyright (c) INRIA 2007-2014 *)
 
 #directory ".";
 #load "pa_macro.cmo";
@@ -246,12 +246,12 @@ value strip_heading_spaces s =
 ;
 
 value strip_char c s = do {
-  let s = String.copy s in
+  let s = string_copy s in
   loop 0 0 where rec loop i j =
     if i = String.length s then String.sub s 0 j
     else if s.[i] = '_' then loop (i + 1) j
     else do {
-      s.[j] := s.[i];
+      string_set s j s.[i];
       loop (i + 1) (j + 1)
     }
 };
