@@ -1,6 +1,6 @@
 (* camlp5r *)
-(* $Id: pr_r.ml,v 6.77 2012-03-09 14:01:54 deraugla Exp $ *)
-(* Copyright (c) INRIA 2007-2012 *)
+(* pr_r.ml,v *)
+(* Copyright (c) INRIA 2007-2014 *)
 
 #directory ".";
 #load "pa_macro.cmo";
@@ -71,8 +71,13 @@ value has_special_chars s =
   if String.length s = 0 then False
   else
     match s.[0] with
-    [ '0'..'9' | 'A'..'Z' | 'a'..'z' | '_' -> False
-    | _ -> True ]
+    | '0'..'9' | 'A'..'Z' | 'a'..'z' | '_' -> False
+    | _ ->
+        match (greek_ascii_equiv s).[0] with
+        | 'A'..'Z' | 'a'..'z' → False
+        | _ → True
+        end
+    end
 ;
 
 value rec is_irrefut_patt =
@@ -1591,7 +1596,7 @@ EXTEND_PRINTER
           let mdl = List.map (fun (m, mt) -> (Pcaml.unvala m, mt)) mdl in
           let rf = if rf then " rec" else "" in
           vlist2 (str_module ("module" ^ rf)) (str_module "and") pc mdl
-      | <:str_item< module type $uid:m$ = $mt$ >> ->
+      | <:str_item< module type $m$ = $mt$ >> ->
           sig_module_or_module_type "module type" '=' pc (m, mt)
       | <:str_item:< open $i$ >> ->
           pprintf pc "open %p" mod_ident (loc, i)
@@ -1642,7 +1647,7 @@ EXTEND_PRINTER
           let rf = if rf then " rec" else "" in
           vlist2 (sig_module_or_module_type ("module" ^ rf) ':')
             (sig_module_or_module_type "and" ':') pc mdl
-      | <:sig_item< module type $uid:m$ = $mt$ >> ->
+      | <:sig_item< module type $m$ = $mt$ >> ->
           sig_module_or_module_type "module type" '=' pc (m, mt)
       | <:sig_item:< open $i$ >> ->
           pprintf pc "open %p" mod_ident (loc, i)
