@@ -1,6 +1,6 @@
 (* camlp5r *)
-(* $Id: pr_scheme.ml,v 6.9 2013-07-02 16:12:43 deraugla Exp $ *)
-(* Copyright (c) INRIA 2007-2012 *)
+(* pr_scheme.ml,v *)
+(* Copyright (c) INRIA 2007-2014 *)
 
 #directory ".";
 #load "pa_macro.cmo";
@@ -1445,12 +1445,12 @@ value copy_to_end src oc first bp =
 
 module Buff =
   struct
-    value buff = ref (String.create 80);
+    value buff = ref (string_create 80);
     value store len x = do {
       if len >= String.length buff.val then
-        buff.val := buff.val ^ String.create (String.length buff.val)
+        buff.val := buff.val ^ string_create (String.length buff.val)
       else ();
-      buff.val.[len] := x;
+      string_set buff.val len x;
       succ len
     };
     value mstore len s =
