@@ -1,11 +1,13 @@
 (* camlp5r *)
-(* $Id: diff.ml,v 6.3 2012-01-09 14:22:21 deraugla Exp $ *)
-(* Copyright (c) INRIA 2007-2012 *)
+(* diff.ml,v *)
+(* Copyright (c) INRIA 2007-2014 *)
 
 (* Parts of Code of GNU diff (analyze.c) translated from C to OCaml
    and adjusted. Basic algorithm described by Eugene W.Myers in:
      "An O(ND) Difference Algorithm and Its Variations" *)
 
+open Versdep;
+
 exception DiagReturn of int;
 
 value diag fd bd sh xv yv xoff xlim yoff ylim = do {
@@ -129,7 +131,7 @@ value make_indexer a b = do {
        try b.(i) := Hashtbl.find htb e with
        [ Not_found -> Hashtbl.add htb e e ])
     b;
-  let ai = Array.create n 0 in
+  let ai = array_create n 0 in
   let k =
     loop 0 0 where rec loop i k =
       if i = n then k
