@@ -1,6 +1,6 @@
 (* camlp5r *)
-(* $Id: grammar.mli,v 6.4 2012-01-09 14:22:21 deraugla Exp $ *)
-(* Copyright (c) INRIA 2007-2012 *)
+(* grammar.mli,v *)
+(* Copyright (c) INRIA 2007-2014 *)
 
 (** Extensible grammars.
 
@@ -201,6 +201,10 @@ value strict_parsing : ref bool;
    (** Flag to apply strict parsing, without trying to recover errors;
        default = [False] *)
 
+value utf8_print : ref bool;
+   (** Flag to consider strings as utf8-encoded when printing them;
+       default = [True] *)
+
 value print_entry : Format.formatter -> Gramext.g_entry 'te -> unit;
    (** General printer for all kinds of entries (obj entries) *)
 
