@@ -1,6 +1,6 @@
 (* camlp5r *)
-(* $Id: plexer.ml,v 6.19 2013-07-03 01:43:10 deraugla Exp $ *)
-(* Copyright (c) INRIA 2007-2012 *)
+(* plexer.ml,v *)
+(* Copyright (c) INRIA 2007-2014 *)
 
 #load "pa_lexer.cmo";
 
@@ -39,10 +39,10 @@ value keyword_or_error ctx loc s =
 ;
 
 value rev_implode l =
-  let s = String.create (List.length l) in
+  let s = string_create (List.length l) in
   loop (String.length s - 1) l where rec loop i =
     fun
-    [ [c :: l] -> do { String.unsafe_set s i c; loop (i - 1) l }
+    [ [c :: l] -> do { string_unsafe_set s i c; loop (i - 1) l }
     | [] -> s ]
 ;
 
