@@ -1,6 +1,6 @@
 (* camlp5r pa_macro.cmo *)
-(* $Id: versdep.ml,v 6.36 2013-06-10 15:57:21 deraugla Exp $ *)
-(* Copyright (c) INRIA 2007-2012 *)
+(* versdep.ml,v *)
+(* Copyright (c) INRIA 2007-2014 *)
 
 open Parsetree;
 open Longident;
@@ -94,22 +94,72 @@ value list_map_check f l =
     | [] -> Some (List.rev rev_l) ]
 ;
 
-value ocaml_value_description t p =
+value ocaml_value_description vn t p =
   IFDEF OCAML_VERSION < OCAML_4_00 THEN {pval_type = t; pval_prim = p}
-  ELSE {pval_type = t; pval_prim = p; pval_loc = t.ptyp_loc} END
+  ELSIFDEF OCAML_VERSION < OCAML_4_02_0 THEN
+    {pval_type = t; pval_prim = p; pval_loc = t.ptyp_loc}
+  ELSE
+    {pval_type = t; pval_prim = p; pval_loc = t.ptyp_loc;
+     pval_name = mkloc t.ptyp_loc vn; pval_attributes = []}
+  END
 ;
 
 value ocaml_class_type_field loc ctfd =
   IFDEF OCAML_VERSION < OCAML_4_00 THEN ctfd
-  ELSE {pctf_desc = ctfd; pctf_loc = loc} END
+  ELSIFDEF OCAML_VERSION < OCAML_4_02_0 THEN
+    {pctf_desc = ctfd; pctf_loc = loc}
+  ELSE
+    {pctf_desc = ctfd; pctf_loc = loc; pctf_attributes = []}
+  END
 ;
 
 value ocaml_class_field loc cfd =
   IFDEF OCAML_VERSION < OCAML_4_00 THEN cfd
-  ELSE {pcf_desc = cfd; pcf_loc = loc} END
+  ELSIFDEF OCAML_VERSION < OCAML_4_02_0 THEN {pcf_desc = cfd; pcf_loc = loc}
+  ELSE {pcf_desc = cfd; pcf_loc = loc; pcf_attributes = []} END
 ;
 
-value ocaml_type_declaration params cl tk pf tm loc variance =
+value ocaml_mktyp loc x =
+  IFDEF OCAML_VERSION < OCAML_4_02_0 THEN {ptyp_desc = x; ptyp_loc = loc}
+  ELSE {ptyp_desc = x; ptyp_loc = loc; ptyp_attributes = []} END
+;
+value ocaml_mkpat loc x =
+  IFDEF OCAML_VERSION < OCAML_4_02_0 THEN {ppat_desc = x; ppat_loc = loc}
+  ELSE {ppat_desc = x; ppat_loc = loc; ppat_attributes = []} END
+;
+value ocaml_mkexp loc x =
+  IFDEF OCAML_VERSION < OCAML_4_02_0 THEN {pexp_desc = x; pexp_loc = loc}
+  ELSE {pexp_desc = x; pexp_loc = loc; pexp_attributes = []} END
+;
+value ocaml_mkmty loc x =
+  IFDEF OCAML_VERSION < OCAML_4_02_0 THEN {pmty_desc = x; pmty_loc = loc}
+  ELSE {pmty_desc = x; pmty_loc = loc; pmty_attributes = []} END
+;
+value ocaml_mkmod loc x =
+  IFDEF OCAML_VERSION < OCAML_4_02_0 THEN {pmod_desc = x; pmod_loc = loc}
+  ELSE {pmod_desc = x; pmod_loc = loc; pmod_attributes = []} END
+;
+value ocaml_mkfield loc (lab, x) fl =
+  IFDEF OCAML_VERSION < OCAML_4_02_0 THEN
+    [{pfield_desc = Pfield lab x; pfield_loc = loc} :: fl]
+  ELSE [(lab, x) :: fl] END
+;
+value ocaml_mkfield_var loc =
+  IFDEF OCAML_VERSION < OCAML_4_02_0 THEN
+    [{pfield_desc = Pfield_var; pfield_loc = loc}]
+  ELSE [] END
+;
+
+IFDEF OCAML_VERSION >= OCAML_4_02_0 THEN
+  value variance_of_bool_bool =
+    fun
+    [ (False, True) -> Contravariant
+    | (True, False) -> Covariant
+    | _ -> Invariant ]
+  ;
+END;
+
+value ocaml_type_declaration tn params cl tk pf tm loc variance =
   IFDEF OCAML_VERSION = OCAML_3_13_0_gadt THEN
     Right
       {ptype_params = params; ptype_cstrs = cl; ptype_kind = tk;
@@ -147,12 +197,28 @@ value ocaml_type_declaration params cl tk pf tm loc variance =
             {ptype_params = params; ptype_cstrs = cl; ptype_kind = tk;
              ptype_private = pf; ptype_manifest = tm; ptype_loc = loc;
              ptype_variance = variance}
-        ELSE
-          let params = List.map (fun os -> Some (mknoloc os)) params in
+        ELSIFDEF OCAML_VERSION < OCAML_4_02_0 THEN
+          let params = List.map (fun os -> Some (mkloc loc os)) params in
           Right
             {ptype_params = params; ptype_cstrs = cl; ptype_kind = tk;
              ptype_private = pf; ptype_manifest = tm; ptype_loc = loc;
              ptype_variance = variance}
+        ELSE
+          let _ =
+            if List.length params <> List.length variance then
+              failwith "internal error: ocaml_type_declaration"
+            else ()
+          in
+          let params =
+            List.map2
+              (fun os va ->
+                 (ocaml_mktyp loc (Ptyp_var os), variance_of_bool_bool va))
+              params variance
+          in
+          Right
+            {ptype_params = params; ptype_cstrs = cl; ptype_kind = tk;
+             ptype_private = pf; ptype_manifest = tm; ptype_loc = loc;
+             ptype_name = mkloc loc tn; ptype_attributes = []}
         END
     | None -> Left "no '_' type param in this ocaml version" ]
   END
@@ -160,22 +226,35 @@ value ocaml_type_declaration params cl tk pf tm loc variance =
 
 value ocaml_class_type =
   IFDEF OCAML_VERSION <= OCAML_1_07 THEN None
-  ELSE Some (fun d loc -> {pcty_desc = d; pcty_loc = loc}) END
+  ELSIFDEF OCAML_VERSION < OCAML_4_02_0 THEN
+    Some (fun d loc -> {pcty_desc = d; pcty_loc = loc})
+  ELSE
+    Some (fun d loc -> {pcty_desc = d; pcty_loc = loc; pcty_attributes = []})
+  END
 ;
 
 value ocaml_class_expr =
   IFDEF OCAML_VERSION <= OCAML_1_07 THEN None
-  ELSE Some (fun d loc -> {pcl_desc = d; pcl_loc = loc}) END
+  ELSIFDEF OCAML_VERSION < OCAML_4_02_0 THEN
+    Some (fun d loc -> {pcl_desc = d; pcl_loc = loc})
+  ELSE
+    Some (fun d loc -> {pcl_desc = d; pcl_loc = loc; pcl_attributes = []})
+  END
 ;
 
 value ocaml_class_structure p cil =
   IFDEF OCAML_VERSION <= OCAML_4_00 THEN (p, cil)
-  ELSE {pcstr_pat = p; pcstr_fields = cil} END
+  ELSIFDEF OCAML_VERSION < OCAML_4_02_0 THEN
+    {pcstr_pat = p; pcstr_fields = cil}
+  ELSE {pcstr_self = p; pcstr_fields = cil} END
 ;
 
 value ocaml_pmty_ident loc li = Pmty_ident (mkloc loc li);
 
-value ocaml_pmty_functor sloc s mt1 mt2 = Pmty_functor (mkloc sloc s) mt1 mt2;
+value ocaml_pmty_functor sloc s mt1 mt2 =
+  IFDEF OCAML_VERSION < OCAML_4_02_0 THEN Pmty_functor (mkloc sloc s) mt1 mt2
+  ELSE Pmty_functor (mkloc sloc s) (Some mt1) mt2 END
+;
 
 value ocaml_pmty_typeof =
   IFDEF OCAML_VERSION < OCAML_3_12 THEN None
@@ -183,8 +262,12 @@ value ocaml_pmty_typeof =
 ;
 
 value ocaml_pmty_with mt lcl =
-  let lcl = List.map (fun (s, c) → (mknoloc s, c)) lcl in
-  Pmty_with mt lcl
+  IFDEF OCAML_VERSION < OCAML_4_02_0 THEN
+    let lcl = List.map (fun (s, c) → (mknoloc s, c)) lcl in
+    Pmty_with mt lcl
+  ELSE
+    let lcl = List.map snd lcl in Pmty_with mt lcl
+  END
 ;
 
 value ocaml_ptype_abstract =
@@ -209,9 +292,16 @@ value ocaml_ptype_record ltl priv =
     Ptype_record ltl priv
   ELSIFDEF OCAML_VERSION < OCAML_4_00 THEN
     Ptype_record ltl
-  ELSE
+  ELSIFDEF OCAML_VERSION < OCAML_4_02_0 THEN
     Ptype_record
       (List.map (fun (s, mf, ct, loc) → (mkloc loc s, mf, ct, loc)) ltl)
+  ELSE
+    Ptype_record
+      (List.map
+         (fun (s, mf, ct, loc) ->
+            {pld_name = mkloc loc s; pld_mutable = mf; pld_type = ct;
+             pld_loc = loc; pld_attributes = []})
+         ltl)
   END
 ;
 
@@ -250,7 +340,7 @@ value ocaml_ptype_variant ctl priv =
             ctl
         in
           Some (Ptype_variant ctl)
-      ELSE
+      ELSIFDEF OCAML_VERSION < OCAML_4_02_0 THEN
         let ctl =
           List.map
             (fun (c, tl, rto, loc) ->
@@ -258,6 +348,17 @@ value ocaml_ptype_variant ctl priv =
             ctl
         in
           Some (Ptype_variant ctl)
+      ELSE
+        let ctl =
+          List.map
+            (fun (c, tl, rto, loc) ->
+               if rto <> None then raise Exit
+               else
+                 {pcd_name = mkloc loc c; pcd_args = tl; pcd_res = None;
+                  pcd_loc = loc; pcd_attributes = []})
+            ctl
+        in
+        Some (Ptype_variant ctl)
       END
     with
     [ Exit -> None ]
@@ -271,10 +372,19 @@ value ocaml_ptyp_arrow lab t1 t2 =
 
 value ocaml_ptyp_class li tl ll =
   IFDEF OCAML_VERSION <= OCAML_2_04 THEN Ptyp_class li tl
-  ELSE Ptyp_class (mknoloc li) tl ll END
+  ELSIFDEF OCAML_VERSION < OCAML_4_02_0 THEN Ptyp_class (mknoloc li) tl ll
+  ELSE Ptyp_class (mknoloc li) tl END
 ;
 
-value ocaml_ptyp_constr li tl = Ptyp_constr (mknoloc li) tl;
+value ocaml_ptyp_constr loc li tl = Ptyp_constr (mkloc loc li) tl;
+
+value ocaml_ptyp_object ml =
+  IFDEF OCAML_VERSION < OCAML_4_02_0 THEN Ptyp_object ml
+  ELSE
+    let ml = List.map (fun (s, t) -> (s, [], t)) ml in
+    Ptyp_object ml Closed
+  END
+;
 
 value ocaml_ptyp_package =
   IFDEF OCAML_VERSION < OCAML_3_12_0 THEN None
@@ -283,7 +393,15 @@ value ocaml_ptyp_package =
 
 value ocaml_ptyp_poly =
   IFDEF OCAML_VERSION <= OCAML_3_04 THEN None
-  ELSE Some (fun cl t -> Ptyp_poly cl t) END
+  ELSIFDEF OCAML_VERSION < OCAML_4_02_0 THEN
+    Some (fun cl t -> Ptyp_poly cl t)
+  ELSE
+    Some
+      (fun cl t ->
+         match cl with
+         [ [] -> t.ptyp_desc
+         | _ -> Ptyp_poly cl t ])
+  END
 ;
 
 value ocaml_ptyp_variant catl clos sl_opt =
@@ -301,7 +419,7 @@ value ocaml_ptyp_variant catl clos sl_opt =
       Some (Ptyp_variant catl clos sl)
     with
     [ Exit -> None ]
-  ELSE
+  ELSIFDEF OCAML_VERSION < OCAML_4_02_0 THEN
     let catl =
       List.map
         (fun
@@ -310,6 +428,16 @@ value ocaml_ptyp_variant catl clos sl_opt =
         catl
     in
     Some (Ptyp_variant catl clos sl_opt)
+  ELSE
+    let catl =
+      List.map
+        (fun
+         [ Left (c, a, tl) -> Rtag c [] a tl
+         | Right t -> Rinherit t ])
+        catl
+    in
+    let clos = if clos then Closed else Open in
+    Some (Ptyp_variant catl clos sl_opt)
   END
 ;
 
@@ -317,6 +445,11 @@ value ocaml_package_type li ltl =
   (mknoloc li, List.map (fun (li, t) → (mkloc t.ptyp_loc li, t)) ltl)
 ;
 
+value ocaml_const_string s =
+  IFDEF OCAML_VERSION < OCAML_4_02_0 THEN Const_string s
+  ELSE Const_string s None END
+;
+
 value ocaml_const_int32 =
   IFDEF OCAML_VERSION <= OCAML_3_06 THEN None
   ELSE Some (fun s -> Const_int32 (Int32.of_string s)) END
@@ -350,7 +483,11 @@ value ocaml_pexp_assertfalse fname loc =
     let bucket = ghexp (Pexp_construct excep (Some triple) False) in
     let raise_ = ghexp (Pexp_ident (Ldot (Lident "Pervasives") "raise")) in
     ocaml_pexp_apply raise_ [("", bucket)]
-  ELSE Pexp_assertfalse END
+  ELSIFDEF OCAML_VERSION < OCAML_4_02_0 THEN Pexp_assertfalse
+  ELSE
+    Pexp_assert
+      (ocaml_mkexp loc (Pexp_construct (mkloc loc (Lident "false")) None))
+  END
 ;
 
 value ocaml_pexp_assert fname loc e =
@@ -377,17 +514,84 @@ value ocaml_pexp_assert fname loc e =
   ELSE Pexp_assert e END
 ;
 
+value ocaml_pexp_constraint e ot1 ot2 =
+  IFDEF OCAML_VERSION < OCAML_4_02_0 THEN Pexp_constraint e ot1 ot2
+  ELSE
+    match ot2 with
+    | Some t2 -> Pexp_coerce e ot1 t2
+    | None ->
+        match ot1 with
+        | Some t1 -> Pexp_constraint e t1
+        | None -> failwith "internal error: ocaml_pexp_constraint"
+        end
+    end
+  END
+;
+
 value ocaml_pexp_construct loc li po chk_arity =
-  Pexp_construct (mkloc loc li) po chk_arity
+  IFDEF OCAML_VERSION < OCAML_4_02_0 THEN
+    Pexp_construct (mkloc loc li) po chk_arity
+  ELSE
+    Pexp_construct (mkloc loc li) po
+  END
+;
+
+value ocaml_pexp_construct_args =
+  IFDEF OCAML_VERSION < OCAML_4_00_0 THEN
+    fun
+    [ Pexp_construct li po chk_arity -> Some (li, 0, po, chk_arity)
+    | _ -> None ]
+  ELSIFDEF OCAML_VERSION < OCAML_4_02_0 THEN
+    fun
+    [ Pexp_construct li po chk_arity -> Some (li.txt, li.loc, po, chk_arity)
+    | _ -> None ]
+  ELSE
+    fun
+    [ Pexp_construct li po -> Some (li.txt, li.loc, po, 0)
+    | _ -> None ]
+  END
+;
+
+value mkexp_ocaml_pexp_construct_arity loc li_loc li al =
+  let a = ocaml_mkexp loc (Pexp_tuple al) in
+  IFDEF OCAML_VERSION < OCAML_4_02_0 THEN
+    ocaml_mkexp loc (ocaml_pexp_construct li_loc li (Some a) True)
+  ELSE
+    {pexp_desc = ocaml_pexp_construct li_loc li (Some a) True;
+     pexp_loc = loc;
+     pexp_attributes = [(mkloc loc "ocaml.explicit_arity", PStr [])]}
+  END
 ;
 
 value ocaml_pexp_field loc e li = Pexp_field e (mkloc loc li);
 
-value ocaml_pexp_for i e1 e2 df e = Pexp_for (mknoloc i) e1 e2 df e;
+value ocaml_pexp_for i e1 e2 df e =
+  IFDEF OCAML_VERSION < OCAML_4_02_0 THEN Pexp_for (mknoloc i) e1 e2 df e
+  ELSE Pexp_for (ocaml_mkpat loc_none (Ppat_var (mknoloc i))) e1 e2 df e END
+;
+
+value ocaml_case (p, wo, loc, e) =
+  IFDEF OCAML_VERSION < OCAML_4_02_0 THEN
+    match wo with
+    | Some w -> (p, ocaml_mkexp loc (Pexp_when w e))
+    | None -> (p, e)
+    end
+  ELSE
+    {pc_lhs = p; pc_guard = wo; pc_rhs = e}
+  END
+;
 
 value ocaml_pexp_function lab eo pel =
   IFDEF OCAML_VERSION <= OCAML_2_04 THEN Pexp_function pel
-  ELSE Pexp_function lab eo pel END
+  ELSIFDEF OCAML_VERSION < OCAML_4_02_0 THEN Pexp_function lab eo pel
+  ELSE
+    match pel with
+    | [{pc_lhs = p; pc_guard = None; pc_rhs = e}] -> Pexp_fun lab eo p e
+    | pel ->
+        if lab = "" && eo = None then Pexp_function pel
+        else failwith "internal error: bad ast in ocaml_pexp_function"
+    end
+  END
 ;
 
 value ocaml_pexp_lazy =
@@ -469,6 +673,11 @@ value ocaml_pexp_variant =
   END
 ;
 
+value ocaml_value_binding loc p e =
+  IFDEF OCAML_VERSION < OCAML_4_02_0 THEN (p, e)
+  ELSE {pvb_pat = p; pvb_expr = e; pvb_loc = loc; pvb_attributes = []} END
+;
+
 value ocaml_ppat_alias p i iloc = Ppat_alias p (mkloc iloc i);
 
 value ocaml_ppat_array =
@@ -476,20 +685,40 @@ value ocaml_ppat_array =
   ELSE Some (fun pl -> Ppat_array pl) END
 ;
 
-value ocaml_ppat_construct li li_loc po chk_arity  =
+value ocaml_ppat_construct loc li po chk_arity  =
   IFDEF OCAML_VERSION < OCAML_4_00 THEN
     Ppat_construct li po chk_arity
+  ELSIFDEF OCAML_VERSION < OCAML_4_02_0 THEN
+    Ppat_construct (mkloc loc li) po chk_arity
   ELSE
-    Ppat_construct (mkloc li_loc li) po chk_arity
+    Ppat_construct (mkloc loc li) po
   END
 ;
 
 value ocaml_ppat_construct_args =
-  fun
-  [ Ppat_construct li po chk_arity ->
-      IFDEF OCAML_VERSION < OCAML_4_00 THEN Some (li, 0, po, chk_arity)
-      ELSE Some (li.txt, li.loc, po, chk_arity) END 
-  | _ -> None ]
+  IFDEF OCAML_VERSION < OCAML_4_02_0 THEN
+    fun
+    [ Ppat_construct li po chk_arity ->
+        IFDEF OCAML_VERSION < OCAML_4_00 THEN Some (li, 0, po, chk_arity)
+        ELSE Some (li.txt, li.loc, po, chk_arity) END 
+    | _ -> None ]
+  ELSE
+    fun
+    [ Ppat_construct li po -> Some (li.txt, li.loc, po, 0)
+    | _ -> None ]
+  END
+;
+
+value mkpat_ocaml_ppat_construct_arity loc li_loc li al =
+  IFDEF OCAML_VERSION < OCAML_4_02_0 THEN
+    let a = ocaml_mkpat loc (Ppat_tuple al) in
+    ocaml_mkpat loc (ocaml_ppat_construct li_loc li (Some a) True)
+  ELSE
+    let a = ocaml_mkpat loc (Ppat_tuple al) in
+    {ppat_desc = ocaml_ppat_construct li_loc li (Some a) True;
+     ppat_loc = loc;
+     ppat_attributes = [(mkloc loc "ocaml.explicit_arity", PStr [])]}
+  END
 ;
 
 value ocaml_ppat_lazy =
@@ -535,80 +764,206 @@ value ocaml_psig_class_type =
   ELSE Some (fun ctl -> Psig_class_type ctl) END
 ;
 
-value ocaml_psig_exception s ed = Psig_exception (mknoloc s) ed;
+value ocaml_psig_exception loc s ed =
+  IFDEF OCAML_VERSION < OCAML_4_02_0 THEN Psig_exception (mkloc loc s) ed
+  ELSE
+    Psig_exception
+      {pext_name = mkloc loc s; pext_kind = Pext_decl ed None;
+       pext_loc = loc; pext_attributes = []}
+  END
+;
 
-value ocaml_psig_module s mt = Psig_module (mknoloc s) mt;
+value ocaml_psig_include loc mt =
+  IFDEF OCAML_VERSION < OCAML_4_02_0 THEN Psig_include mt
+  ELSE
+    Psig_include {pincl_mod = mt; pincl_loc = loc; pincl_attributes = []}
+  END
+;
 
-value ocaml_psig_modtype s mtd = Psig_modtype (mknoloc s) mtd;
+value ocaml_psig_module loc s mt =
+  IFDEF OCAML_VERSION < OCAML_4_02_0 THEN Psig_module (mknoloc s) mt
+  ELSE
+    Psig_module
+      {pmd_name = mkloc loc s; pmd_type = mt; pmd_attributes = [];
+       pmd_loc = loc}
+  END
+;
 
-value ocaml_psig_open li =
-  IFDEF OCAML_VERSION < OCAML_4_01 THEN Psig_open (mknoloc li)
-  ELSE Psig_open Fresh (mknoloc li) END
+value ocaml_psig_modtype loc s mto =
+  IFDEF OCAML_VERSION < OCAML_4_02_0 THEN
+    let mtd =
+      match mto with
+      | None -> Pmodtype_abstract
+      | Some t -> Pmodtype_manifest t
+      end
+    in
+    Psig_modtype (mknoloc s) mtd
+  ELSE
+    let pmtd =
+      {pmtd_name = mkloc loc s; pmtd_type = mto; pmtd_attributes = [];
+       pmtd_loc = loc}
+    in
+    Psig_modtype pmtd
+  END
+;
+
+value ocaml_psig_open loc li =
+  IFDEF OCAML_VERSION < OCAML_4_01 THEN Psig_open (mkloc loc li)
+  ELSIFDEF OCAML_VERSION < OCAML_4_02_0 THEN Psig_open Fresh (mkloc loc li)
+  ELSE
+    Psig_open
+      {popen_lid = mknoloc li; popen_override = Fresh; popen_loc = loc;
+       popen_attributes = []}
+  END
 ;
 
 value ocaml_psig_recmodule =
   IFDEF OCAML_VERSION <= OCAML_3_06 THEN None
-  ELSE
+  ELSIFDEF OCAML_VERSION < OCAML_4_02_0 THEN
     let f ntl =
       let ntl = List.map (fun (s, mt) → (mknoloc s, mt)) ntl in
       Psig_recmodule ntl
     in
     Some f
+  ELSE
+    let f ntl =
+      let ntl =
+        List.map
+          (fun (s, mt) ->
+             {pmd_name = mknoloc s; pmd_type = mt; pmd_attributes = [];
+              pmd_loc = loc_none})
+          ntl
+      in
+      Psig_recmodule ntl
+    in
+    Some f
   END
 ;
 
 value ocaml_psig_type stl =
-  let stl = List.map (fun (s, t) → (mknoloc s, t)) stl in
-  Psig_type stl
+  IFDEF OCAML_VERSION < OCAML_4_02_0 THEN
+    let stl = List.map (fun (s, t) → (mknoloc s, t)) stl in
+    Psig_type stl
+  ELSE
+    let stl = List.map (fun (s, t) -> t) stl in Psig_type stl
+  END
 ;
 
-value ocaml_psig_value s vd = Psig_value (mknoloc s) vd;
+value ocaml_psig_value s vd =
+  IFDEF OCAML_VERSION < OCAML_4_02_0 THEN Psig_value (mknoloc s) vd
+  ELSE Psig_value vd END
+;
 
 value ocaml_pstr_class_type =
   IFDEF OCAML_VERSION <= OCAML_1_07 THEN None
   ELSE Some (fun ctl -> Pstr_class_type ctl) END
 ;
 
-value ocaml_pstr_exception s ed = Pstr_exception (mknoloc s) ed;
+value ocaml_pstr_eval e =
+  IFDEF OCAML_VERSION < OCAML_4_02_0 THEN Pstr_eval e
+  ELSE Pstr_eval e [] END
+;
+
+value ocaml_pstr_exception loc s ed =
+  IFDEF OCAML_VERSION < OCAML_4_02_0 THEN Pstr_exception (mkloc loc s) ed
+  ELSE
+    Pstr_exception
+      {pext_name = mkloc loc s; pext_kind = Pext_decl ed None;
+       pext_loc = loc; pext_attributes = []}
+  END
+;
 
 value ocaml_pstr_exn_rebind =
   IFDEF OCAML_VERSION <= OCAML_2_99 THEN None
-  ELSE Some (fun s li -> Pstr_exn_rebind (mknoloc s) (mknoloc li)) END
+  ELSIFDEF OCAML_VERSION < OCAML_4_02_0 THEN
+    Some (fun loc s li -> Pstr_exn_rebind (mkloc loc s) (mkloc loc li))
+  ELSE
+    Some
+      (fun loc s li ->
+         Pstr_exception
+           {pext_name = mkloc loc s; pext_kind = Pext_rebind (mkloc loc li);
+            pext_loc = loc; pext_attributes = []})
+  END
 ;
 
 value ocaml_pstr_include =
   IFDEF OCAML_VERSION <= OCAML_3_00 THEN None
-  ELSE Some (fun me -> Pstr_include me) END
+  ELSIFDEF OCAML_VERSION < OCAML_4_02_0 THEN
+    Some (fun loc me -> Pstr_include me)
+  ELSE
+    Some
+      (fun loc me ->
+         Pstr_include
+           {pincl_mod = me; pincl_loc = loc; pincl_attributes = []})
+  END
 ;
 
-value ocaml_pstr_modtype s mt = Pstr_modtype (mknoloc s) mt;
+value ocaml_pstr_modtype loc s mt =
+  IFDEF OCAML_VERSION < OCAML_4_02_0 THEN Pstr_modtype (mkloc loc s) mt
+  ELSE
+    let pmtd =
+      {pmtd_name = mkloc loc s; pmtd_type = Some mt; pmtd_attributes = [];
+       pmtd_loc = loc}
+    in
+    Pstr_modtype pmtd
+  END
+;
 
-value ocaml_pstr_module s me = Pstr_module (mknoloc s) me;
+value ocaml_pstr_module loc s me =
+  IFDEF OCAML_VERSION < OCAML_4_02_0 THEN Pstr_module (mkloc loc s) me
+  ELSE
+    let mb =
+      {pmb_name = mkloc loc s; pmb_expr = me; pmb_attributes = [];
+       pmb_loc = loc}
+    in
+    Pstr_module mb
+  END
+;
 
-value ocaml_pstr_open li =
+value ocaml_pstr_open loc li =
   IFDEF OCAML_VERSION < OCAML_4_01 THEN Pstr_open (mknoloc li)
-  ELSE Pstr_open Fresh (mknoloc li) END
+  ELSIFDEF OCAML_VERSION < OCAML_4_02_0 THEN Pstr_open Fresh (mknoloc li)
+  ELSE
+    Pstr_open
+      {popen_lid = mknoloc li; popen_override = Fresh; popen_loc = loc;
+       popen_attributes = []}
+  END
 ;
 
-value ocaml_pstr_primitive s vd = Pstr_primitive (mknoloc s) vd;
+value ocaml_pstr_primitive s vd =
+  IFDEF OCAML_VERSION < OCAML_4_02_0 THEN Pstr_primitive (mknoloc s) vd
+  ELSE Pstr_primitive vd END
+;
 
 value ocaml_pstr_recmodule =
   IFDEF OCAML_VERSION <= OCAML_3_06 THEN None
   ELSIFDEF OCAML_VERSION < OCAML_4_00 THEN
     Some (fun nel -> Pstr_recmodule nel)
-  ELSE
+  ELSIFDEF OCAML_VERSION < OCAML_4_02_0 THEN
     let f nel =
       Pstr_recmodule (List.map (fun (s, mt, me) → (mknoloc s, mt, me)) nel)
     in
     Some f
+  ELSE
+    let f nel =
+      Pstr_recmodule
+        (List.map
+           (fun (s, mt, me) ->
+              {pmb_name = mknoloc s; pmb_expr = me; pmb_attributes = [];
+               pmb_loc = loc_none})
+           nel)
+    in
+    Some f
   END
 ;
 
 value ocaml_pstr_type stl = 
   IFDEF OCAML_VERSION < OCAML_4_00 THEN Pstr_type stl
-  ELSE
+  ELSIFDEF OCAML_VERSION < OCAML_4_02_0 THEN
     let stl = List.map (fun (s, t) → (mknoloc s, t)) stl in
     Pstr_type stl
+  ELSE
+    let stl = List.map (fun (s, t) -> t) stl in Pstr_type stl
   END
 ;
 
@@ -619,18 +974,37 @@ value ocaml_class_infos =
       (fun virt params name expr loc variance ->
          {pci_virt = virt; pci_params = params; pci_name = name;
           pci_expr = expr; pci_loc = loc})
-  ELSE
+  ELSIFDEF OCAML_VERSION < OCAML_4_02_0 THEN
     Some
       (fun virt (sl, sloc) name expr loc variance ->
         let params = (List.map (fun s → mkloc loc s) sl, sloc) in
         {pci_virt = virt; pci_params = params; pci_name = mkloc loc name;
          pci_expr = expr; pci_loc = loc; pci_variance = variance})
+  ELSE
+    Some
+      (fun virt (sl, sloc) name expr loc variance ->
+         let _ =
+           if List.length sl <> List.length variance then
+             failwith "internal error: ocaml_class_infos"
+           else ()
+         in
+         let params =
+           List.map2
+            (fun os va ->
+               (ocaml_mktyp loc (Ptyp_var os), variance_of_bool_bool va))
+            sl variance
+         in
+         {pci_virt = virt; pci_params = params; pci_name = mkloc loc name;
+          pci_expr = expr; pci_loc = loc; pci_attributes = []})
   END
 ;
 
 value ocaml_pmod_ident li = Pmod_ident (mknoloc li);
 
-value ocaml_pmod_functor s mt me = Pmod_functor (mknoloc s) mt me;
+value ocaml_pmod_functor s mt me =
+  IFDEF OCAML_VERSION < OCAML_4_02_0 THEN Pmod_functor (mknoloc s) mt me
+  ELSE Pmod_functor (mknoloc s) (Some mt) me END
+;
 
 value ocaml_pmod_unpack =
   IFDEF OCAML_VERSION < OCAML_3_12 THEN None
@@ -646,41 +1020,58 @@ value ocaml_pcf_cstr =
   IFDEF OCAML_VERSION <= OCAML_1_07 THEN None
   ELSIFDEF OCAML_VERSION < OCAML_4_00 THEN
     Some (fun (t1, t2, loc) -> Pcf_cstr (t1, t2, loc))
-  ELSE Some (fun (t1, t2, loc) -> Pcf_constr (t1, t2)) END
+  ELSIFDEF OCAML_VERSION < OCAML_4_02_0 THEN
+    Some (fun (t1, t2, loc) -> Pcf_constr (t1, t2))
+  ELSE
+    Some (fun (t1, t2, loc) -> Pcf_constraint (t1, t2))
+  END
 ;
 
 value ocaml_pcf_inher =
   IFDEF OCAML_VERSION <= OCAML_1_07 THEN
     fun (id, cl, el, loc) pb -> Pcf_inher (id, cl, el, pb, loc)
-  ELSIFDEF OCAML_VERSION >= OCAML_3_12 THEN
+  ELSIFDEF OCAML_VERSION < OCAML_3_12 THEN
+    fun ce pb -> Pcf_inher ce pb
+  ELSIFDEF OCAML_VERSION < OCAML_4_02_0 THEN
     fun ce pb -> Pcf_inher Fresh ce pb
   ELSE
-    fun ce pb -> Pcf_inher ce pb
+    fun ce pb -> Pcf_inherit Fresh ce pb
   END
 ;
 
 value ocaml_pcf_init =
   IFDEF OCAML_VERSION <= OCAML_1_07 THEN None
-  ELSE Some (fun e -> Pcf_init e) END
+  ELSIFDEF OCAML_VERSION < OCAML_4_02_0 THEN Some (fun e -> Pcf_init e)
+  ELSE Some (fun e -> Pcf_initializer e) END
 ;
 
 value ocaml_pcf_meth (s, pf, ovf, e, loc) =
   let pf = if pf then Private else Public in
-  IFDEF OCAML_VERSION >= OCAML_3_12 THEN
+  IFDEF OCAML_VERSION < OCAML_3_12 THEN Pcf_meth (s, pf, e, loc)
+  ELSE
     let ovf = if ovf then Override else Fresh in
     IFDEF OCAML_VERSION < OCAML_4_00 THEN Pcf_meth (s, pf, ovf, e, loc)
-    ELSE Pcf_meth (mkloc loc s, pf, ovf, e) END
-  ELSE Pcf_meth (s, pf, e, loc) END
+    ELSIFDEF OCAML_VERSION < OCAML_4_02_0 THEN
+      Pcf_meth (mkloc loc s, pf, ovf, e)
+    ELSE
+      Pcf_method (mkloc loc s, pf, Cfk_concrete ovf e)
+    END
+  END
 ;
 
 value ocaml_pcf_val (s, mf, ovf, e, loc) =
   let mf = if mf then Mutable else Immutable in
   IFDEF OCAML_VERSION <= OCAML_1_07 THEN Pcf_val (s, Public, mf, Some e, loc)
-  ELSIFDEF OCAML_VERSION >= OCAML_3_12 THEN
+  ELSIFDEF OCAML_VERSION < OCAML_3_12 THEN Pcf_val (s, mf, e,  loc)
+  ELSE
     let ovf = if ovf then Override else Fresh in
     IFDEF OCAML_VERSION < OCAML_4_00 THEN Pcf_val (s, mf, ovf, e, loc)
-    ELSE Pcf_val (mkloc loc s, mf, ovf, e) END
-  ELSE Pcf_val (s, mf, e,  loc) END
+    ELSIFDEF OCAML_VERSION < OCAML_4_02_0 THEN
+      Pcf_val (mkloc loc s, mf, ovf, e)
+    ELSE
+      Pcf_val (mkloc loc s, mf, Cfk_concrete ovf e)
+    END
+  END
 ;
 
 value ocaml_pcf_valvirt =
@@ -689,7 +1080,9 @@ value ocaml_pcf_valvirt =
     let ocaml_pcf (s, mf, t, loc) =
       let mf = if mf then Mutable else Immutable in
       IFDEF OCAML_VERSION < OCAML_4_00 THEN Pcf_valvirt (s, mf, t, loc)
-      ELSE Pcf_valvirt (mkloc loc s, mf, t) END
+      ELSIFDEF OCAML_VERSION < OCAML_4_02_0 THEN
+        Pcf_valvirt (mkloc loc s, mf, t)
+      ELSE Pcf_val (mkloc loc s, mf, Cfk_virtual t) END
     in
     Some ocaml_pcf
   END
@@ -697,7 +1090,8 @@ value ocaml_pcf_valvirt =
 
 value ocaml_pcf_virt (s, pf, t, loc) =
   IFDEF OCAML_VERSION < OCAML_4_00 THEN Pcf_virt (s, pf, t, loc)
-  ELSE Pcf_virt (mkloc loc s, pf, t) END
+  ELSIFDEF OCAML_VERSION < OCAML_4_02_0 THEN Pcf_virt (mkloc loc s, pf, t)
+  ELSE Pcf_val (mkloc loc s, Immutable, Cfk_virtual t) END
 ;
 
 value ocaml_pcl_apply =
@@ -743,13 +1137,22 @@ value ocaml_pctf_cstr =
   IFDEF OCAML_VERSION <= OCAML_1_07 THEN None
   ELSIFDEF OCAML_VERSION < OCAML_4_00 THEN
     Some (fun (t1, t2, loc) -> Pctf_cstr (t1, t2, loc))
+  ELSIFDEF OCAML_VERSION < OCAML_4_02_0 THEN
+    Some (fun (t1, t2, loc) -> Pctf_cstr (t1, t2))
   ELSE
-    Some (fun (t1, t2, loc) -> Pctf_cstr (t1, t2)) END
+    Some (fun (t1, t2, loc) -> Pctf_constraint (t1, t2))
+  END
+;
+
+value ocaml_pctf_inher ct =
+  IFDEF OCAML_VERSION < OCAML_4_02_0 THEN Pctf_inher ct
+  ELSE Pctf_inherit ct END
 ;
 
 value ocaml_pctf_meth (s, pf, t, loc) =
   IFDEF OCAML_VERSION < OCAML_4_00 THEN Pctf_meth (s, pf, t, loc)
-  ELSE Pctf_meth (s, pf, t) END
+  ELSIFDEF OCAML_VERSION < OCAML_4_02_0 THEN Pctf_meth (s, pf, t)
+  ELSE Pctf_method (s, pf, Concrete, t) END
 ;
 
 value ocaml_pctf_val (s, mf, t, loc) =
@@ -761,7 +1164,8 @@ value ocaml_pctf_val (s, mf, t, loc) =
 
 value ocaml_pctf_virt (s, pf, t, loc) =
   IFDEF OCAML_VERSION < OCAML_4_00 THEN Pctf_virt (s, pf, t, loc)
-  ELSE Pctf_virt (s, pf, t) END
+  ELSIFDEF OCAML_VERSION < OCAML_4_02_0 THEN Pctf_virt (s, pf, t)
+  ELSE Pctf_val (s, Immutable, Virtual, t) END
 ;
 
 value ocaml_pcty_constr =
@@ -774,8 +1178,11 @@ value ocaml_pcty_fun =
     None
   ELSIFDEF OCAML_VERSION <= OCAML_2_04 THEN
     Some (fun lab t ct -> Pcty_fun t ct)
+  ELSIFDEF OCAML_VERSION < OCAML_4_02_0 THEN
+    Some (fun lab t ct -> Pcty_fun lab t ct)
   ELSE
-    Some (fun lab t ct -> Pcty_fun lab t ct) END
+    Some (fun lab t ct -> Pcty_arrow lab t ct)
+  END
 ;
 
 value ocaml_pcty_signature =
@@ -785,7 +1192,11 @@ value ocaml_pcty_signature =
   ELSE
     let f (t, ctfl) =
       let cs =
-        {pcsig_self = t; pcsig_fields = ctfl; pcsig_loc = t.ptyp_loc}
+        IFDEF OCAML_VERSION < OCAML_4_02_0 THEN
+          {pcsig_self = t; pcsig_fields = ctfl; pcsig_loc = t.ptyp_loc}
+        ELSE
+          {pcsig_self = t; pcsig_fields = ctfl}
+        END
       in
       Pcty_signature cs
     in
@@ -800,10 +1211,22 @@ value ocaml_pdir_bool =
 
 value ocaml_pwith_modsubst =
   IFDEF OCAML_VERSION < OCAML_3_12_0 THEN None
-  ELSE Some (fun loc me -> Pwith_modsubst (mkloc loc me)) END
+  ELSIFDEF OCAML_VERSION < OCAML_4_02_0 THEN
+    Some (fun loc me -> Pwith_modsubst (mkloc loc me))
+  ELSE
+    Some (fun loc me -> Pwith_modsubst (mkloc loc "") (mkloc loc me))
+  END
 ;
 
-value ocaml_pwith_module loc me = Pwith_module (mkloc loc me);
+value ocaml_pwith_type loc (i, td) =
+  IFDEF OCAML_VERSION < OCAML_4_02_0 THEN Pwith_type td
+  ELSE Pwith_type (mkloc loc i) td END
+;
+
+value ocaml_pwith_module loc me =
+  IFDEF OCAML_VERSION < OCAML_4_02_0 THEN Pwith_module (mkloc loc me)
+  ELSE Pwith_module (mkloc loc (Lident "")) (mkloc loc me) END
+;
 
 value ocaml_pwith_typesubst =
   IFDEF OCAML_VERSION < OCAML_3_12_0 THEN None
@@ -1009,3 +1432,28 @@ value string_contains =
     String.contains
   END
 ;
+
+value string_copy =
+  IFDEF OCAML_VERSION < OCAML_4_02_0 THEN String.copy
+  ELSE Bytes.copy END
+;
+
+value string_create =
+  IFDEF OCAML_VERSION < OCAML_4_02_0 THEN String.create
+  ELSE Bytes.create END
+;
+
+value string_unsafe_set =
+  IFDEF OCAML_VERSION < OCAML_4_02_0 THEN String.unsafe_set
+  ELSE Bytes.unsafe_set END
+;
+
+value string_set =
+  IFDEF OCAML_VERSION < OCAML_4_02_0 THEN String.set
+  ELSE Bytes.set END
+;
+
+value array_create =
+  IFDEF OCAML_VERSION < OCAML_4_02_0 THEN Array.create
+  ELSE Array.make END
+;
