@@ -1,5 +1,5 @@
 (* camlp5r *)
-(* $Id: ast2pt.ml,v 6.53 2013-03-19 15:25:22 deraugla Exp $ *)
+(* ast2pt.ml,v *)
 
 #load "q_MLast.cmo";
 
@@ -62,14 +62,16 @@ value mkloc loc =
   ocaml_location (glob_fname.val, lnum, bolp, lnuml, bolpl, bp, ep)
 ;
 
-value mktyp loc d = {ptyp_desc = d; ptyp_loc = mkloc loc};
-value mkpat loc d = {ppat_desc = d; ppat_loc = mkloc loc};
-value mkexp loc d = {pexp_desc = d; pexp_loc = mkloc loc};
-value mkmty loc d = {pmty_desc = d; pmty_loc = mkloc loc};
+value mktyp loc d = ocaml_mktyp (mkloc loc) d;
+value mkpat loc d = ocaml_mkpat (mkloc loc) d;
+value mkexp loc d = ocaml_mkexp (mkloc loc) d;
+value mkmty loc d = ocaml_mkmty (mkloc loc) d;
 value mksig loc d = {psig_desc = d; psig_loc = mkloc loc};
-value mkmod loc d = {pmod_desc = d; pmod_loc = mkloc loc};
+value mkmod loc d = ocaml_mkmod (mkloc loc) d;
 value mkstr loc d = {pstr_desc = d; pstr_loc = mkloc loc};
-value mkfield loc d = {pfield_desc = d; pfield_loc = mkloc loc};
+value mkfield loc d fl = ocaml_mkfield (mkloc loc) d fl;
+value mkfield_var loc = ocaml_mkfield_var (mkloc loc);
+
 value mkcty loc d =
   match ocaml_class_type with
   [ Some class_type → class_type d (mkloc loc)
@@ -87,9 +89,10 @@ value mklazy loc e =
       let ghpat = mkpat loc in
       let ghexp = mkexp loc in
       let void_pat =
-        ghpat (ocaml_ppat_construct (Lident "()") (mkloc loc) None False)
+        ghpat (ocaml_ppat_construct (mkloc loc) (Lident "()") None False)
       in
-      let f = ghexp (ocaml_pexp_function "" None [(void_pat, e)]) in
+      let pwe = ocaml_case (void_pat, None, mkloc loc, e) in
+      let f = ghexp (ocaml_pexp_function "" None [pwe]) in
       let delayed = Ldot (Lident "Lazy") "Delayed" in
       let cloc = mkloc loc in
       let df = ghexp (ocaml_pexp_construct cloc delayed (Some f) False) in
@@ -100,11 +103,11 @@ value mklazy loc e =
 value strip_char c s =
   match try Some (String.index s c) with [ Not_found → None ] with
   [ Some _ →
-      let s = String.copy s in
+      let s = string_copy s in
       loop 0 0 where rec loop i j =
         if i = String.length s then String.sub s 0 j
         else if s.[i] = '_' then loop (i + 1) j
-        else do { s.[j] := s.[i]; loop (i + 1) (j + 1) }
+        else do { string_set s j s.[i]; loop (i + 1) (j + 1) }
   | None → s ]
 ;
 
@@ -240,7 +243,7 @@ value rec ctyp =
   [ TyAcc loc _ _ as f →
       let (is_cls, li) = ctyp_long_id f in
       if is_cls then mktyp loc (ocaml_ptyp_class li [] [])
-      else mktyp loc (ocaml_ptyp_constr li [])
+      else mktyp loc (ocaml_ptyp_constr (mkloc loc) li [])
   | TyAli loc t1 t2 →
       let (t, i) =
         match (t1, t2) with
@@ -254,7 +257,7 @@ value rec ctyp =
       let (f, al) = ctyp_fa [] f in
       let (is_cls, li) = ctyp_long_id f in
       if is_cls then mktyp loc (ocaml_ptyp_class li (List.map ctyp al) [])
-      else mktyp loc (ocaml_ptyp_constr li (List.map ctyp al))
+      else mktyp loc (ocaml_ptyp_constr (mkloc loc) li (List.map ctyp al))
   | TyArr loc (TyLab loc1 lab t1) t2 →
       mktyp loc (ocaml_ptyp_arrow (uv lab) (ctyp t1) (ctyp t2))
   | TyArr loc (TyOlb loc1 lab t1) t2 →
@@ -264,11 +267,11 @@ value rec ctyp =
       in
       mktyp loc (ocaml_ptyp_arrow ("?" ^ uv lab) (ctyp t1) (ctyp t2))
   | TyArr loc t1 t2 → mktyp loc (ocaml_ptyp_arrow "" (ctyp t1) (ctyp t2))
-  | TyObj loc fl v → mktyp loc (Ptyp_object (meth_list loc (uv fl) v))
+  | TyObj loc fl v → mktyp loc (ocaml_ptyp_object (meth_list loc (uv fl) v))
   | TyCls loc id →
       mktyp loc (ocaml_ptyp_class (long_id_of_string_list loc (uv id)) [] [])
   | TyLab loc _ _ → error loc "labeled type not allowed here"
-  | TyLid loc s → mktyp loc (ocaml_ptyp_constr (Lident (uv s)) [])
+  | TyLid loc s → mktyp loc (ocaml_ptyp_constr (mkloc loc) (Lident (uv s)) [])
   | TyMan loc _ _ _ → error loc "type manifest not allowed here"
   | TyOlb loc lab _ → error loc "labeled type not allowed here"
   | TyPck loc mt →
@@ -286,7 +289,7 @@ value rec ctyp =
   | TyRec loc _ → error loc "record type not allowed here"
   | TySum loc _ → error loc "sum type not allowed here"
   | TyTup loc tl → mktyp loc (Ptyp_tuple (List.map ctyp (uv tl)))
-  | TyUid loc s → mktyp loc (ocaml_ptyp_constr (Lident (uv s)) [])
+  | TyUid loc s → mktyp loc (ocaml_ptyp_constr (mkloc loc) (Lident (uv s)) [])
   | TyVrn loc catl ool →
       let catl =
         List.map
@@ -309,9 +312,9 @@ value rec ctyp =
   | TyXtr loc _ _ → error loc "bad ast TyXtr" ]
 and meth_list loc fl v =
   match fl with
-  [ [] → if uv v then [mkfield loc Pfield_var] else []
+  [ [] → if uv v then mkfield_var loc else []
   | [(lab, t) :: fl] →
-      [mkfield loc (Pfield lab (add_polytype t)) :: meth_list loc fl v] ]
+      mkfield loc (lab, add_polytype t) (meth_list loc fl v) ]
 and add_polytype t =
   match ocaml_ptyp_poly with
   [ Some ptyp_poly →
@@ -363,11 +366,11 @@ value variance_of_var =
   | None → (False, False) ]
 ;
 
-value mktype loc tl cl tk pf tm =
+value mktype loc tn tl cl tk pf tm =
   let (params, var_list) = List.split tl in
   let variance = List.map variance_of_var var_list in
   let params = List.map uv params in
-  match ocaml_type_declaration params cl tk pf tm (mkloc loc) variance with
+  match ocaml_type_declaration tn params cl tk pf tm (mkloc loc) variance with
   [ Right td → td
   | Left msg → error loc msg ]
 ;
@@ -407,18 +410,18 @@ value mktvariant loc ctl priv =
   | None → error loc "no generalized data types in this ocaml version" ]
 ;
 
-value type_decl tl priv cl =
+value type_decl tn tl priv cl =
   fun
   [ TyMan loc t pf <:ctyp< { $list:ltl$ } >> →
       let priv = if uv pf then Private else Public in
-      mktype loc tl cl (mktrecord ltl (uv pf)) priv (Some (ctyp t))
+      mktype loc tn tl cl (mktrecord ltl (uv pf)) priv (Some (ctyp t))
   | TyMan loc t pf <:ctyp< [ $list:ctl$ ] >> →
       let priv = if uv pf then Private else Public in
-      mktype loc tl cl (mktvariant loc ctl (uv pf)) priv (Some (ctyp t))
+      mktype loc tn tl cl (mktvariant loc ctl (uv pf)) priv (Some (ctyp t))
   | TyRec loc ltl →
-      mktype loc tl cl (mktrecord (uv ltl) False) priv None
+      mktype loc tn tl cl (mktrecord (uv ltl) False) priv None
   | TySum loc ctl →
-      mktype loc tl cl (mktvariant loc (uv ctl) False) priv None
+      mktype loc tn tl cl (mktvariant loc (uv ctl) False) priv None
   | t →
       let m =
         match t with
@@ -427,10 +430,10 @@ value type_decl tl priv cl =
             else None
         | _ → Some (ctyp t) ]
       in
-      mktype (loc_of_ctyp t) tl cl Ptype_abstract priv m ]
+      mktype (loc_of_ctyp t) tn tl cl Ptype_abstract priv m ]
 ;
 
-value mkvalue_desc t p = ocaml_value_description (ctyp t) p;
+value mkvalue_desc vn t p = ocaml_value_description vn (ctyp t) p;
 
 value option f =
   fun
@@ -456,14 +459,14 @@ value rec module_expr_long_id =
   | t → error (loc_of_module_expr t) "bad module expr long ident" ]
 ;
 
-value type_decl_of_with_type loc tpl pf ct =
+value type_decl_of_with_type loc tn tpl pf ct =
   let (params, var_list) = List.split (uv tpl) in
   let variance = List.map variance_of_var var_list in
   let params = List.map uv params in
   let ct = Some (ctyp ct) in
   let tk = if pf then ocaml_ptype_abstract else Ptype_abstract in
   let pf = if pf then Private else Public in
-  ocaml_type_declaration params [] tk pf ct (mkloc loc) variance
+  ocaml_type_declaration tn params [] tk pf ct (mkloc loc) variance
 ;
 
 value mkwithc =
@@ -478,13 +481,14 @@ value mkwithc =
            pwith_modsubst (mkloc loc) (module_expr_long_id m))
       | None → error loc "no with module := in this ocaml version" ]
   | WcTyp loc id tpl pf ct →
-      match type_decl_of_with_type loc tpl (uv pf) ct with
-      [ Right td → (long_id_of_string_list loc (uv id), Pwith_type td)
+      let li = long_id_of_string_list loc (uv id) in
+      match type_decl_of_with_type loc "" tpl (uv pf) ct with
+      [ Right td -> (li, ocaml_pwith_type (mkloc loc) (li, td))
       | Left msg → error loc msg ]
   | WcTys loc id tpl t →
       match ocaml_pwith_typesubst with
       [ Some pwith_typesubst →
-          match type_decl_of_with_type loc tpl False t with
+          match type_decl_of_with_type loc "" tpl False t with
           [ Right td →
               let li = long_id_of_string_list loc (uv id) in
               (li, pwith_typesubst td)
@@ -533,8 +537,8 @@ value rec patt =
         [ (<:patt:< $uid:i$ >>, il) →
             match p2 with
             [ <:patt< $uid:s$ >> →
-                ocaml_ppat_construct (mkli (conv_con s) [i :: il])
-                  (mkloc loc) None (not Prtools.no_constructors_arity.val)
+                ocaml_ppat_construct (mkloc loc) (mkli (conv_con s) [i :: il])
+                  None (not Prtools.no_constructors_arity.val)
             | _ → error (loc_of_patt p2) "bad access pattern" ]
         | _ → error (loc_of_patt p2) "bad pattern" ]
       in
@@ -558,13 +562,11 @@ value rec patt =
           if Prtools.no_constructors_arity.val then
             let a =
               match al with
-              [ [a] → a
-              | _ → mkpat loc (Ppat_tuple al) ]
+              [ [a] -> a
+              | _ -> mkpat loc (Ppat_tuple al) ]
             in
-            mkpat loc (ocaml_ppat_construct li li_loc (Some a) False)
-          else
-            let a = mkpat loc (Ppat_tuple al) in
-            mkpat loc (ocaml_ppat_construct li li_loc (Some a) True)
+            mkpat loc (ocaml_ppat_construct li_loc li (Some a) False)
+          else mkpat_ocaml_ppat_construct_arity (mkloc loc) li_loc li al
       | Some _ | None →
           match ocaml_ppat_variant with
           [ Some (ppat_variant_pat, ppat_variant) →
@@ -620,7 +622,8 @@ value rec patt =
       mkpat loc (ocaml_ppat_record (List.map mklabpat lpl) is_closed)
   | PaStr loc s →
       mkpat loc
-        (Ppat_constant (Const_string (string_of_string_token loc (uv s))))
+        (Ppat_constant
+           (ocaml_const_string (string_of_string_token loc (uv s))))
   | PaTup loc pl → mkpat loc (Ppat_tuple (List.map patt (uv pl)))
   | PaTyc loc p t → mkpat loc (Ppat_constraint (patt p) (ctyp t))
   | PaTyp loc sl →
@@ -632,7 +635,7 @@ value rec patt =
   | PaUid loc s →
       let ca = not Prtools.no_constructors_arity.val in
       mkpat loc
-        (ocaml_ppat_construct (Lident (conv_con (uv s))) (mkloc loc) None ca)
+        (ocaml_ppat_construct (mkloc loc) (Lident (conv_con (uv s))) None ca)
   | PaUnp loc s mto →
       match ocaml_ppat_unpack with
       [ Some (ppat_unpack, ptyp_package) →
@@ -788,20 +791,20 @@ value rec expr =
         | _ → f ]
       in
       let al = List.rev (List.fold_left label_expr [] al) in
-      match (expr f).pexp_desc with
-      [ Pexp_construct li None _ →
+      match ocaml_pexp_construct_args (expr f).pexp_desc with
+      [ Some (li, li_loc, None, _) →
           let al = List.map snd al in
           if Prtools.no_constructors_arity.val then
             let a =
               match al with
-              [ [a] → a
-              | _ → mkexp loc (Pexp_tuple al) ]
+              [ [a] -> a
+              | _ -> mkexp loc (Pexp_tuple al) ]
             in
-            mkexp loc (Pexp_construct li (Some a) False)
+            mkexp loc (ocaml_pexp_construct li_loc li (Some a) False)
           else
-            let a = mkexp loc (Pexp_tuple al) in
-            mkexp loc (Pexp_construct li (Some a) True)
-      | e →
+            mkexp_ocaml_pexp_construct_arity (mkloc loc) li_loc li al
+      | Some _ | None →
+          let e = (expr f).pexp_desc in
           match ocaml_pexp_variant with
           [ Some (pexp_variant_pat, pexp_variant) →
               match pexp_variant_pat e with
@@ -861,7 +864,8 @@ value rec expr =
   | ExChr loc s →
       mkexp loc (Pexp_constant (Const_char (char_of_char_token loc (uv s))))
   | ExCoe loc e t1 t2 →
-      mkexp loc (Pexp_constraint (expr e) (option ctyp t1) (Some (ctyp t2)))
+      mkexp loc
+        (ocaml_pexp_constraint (expr e) (option ctyp t1) (Some (ctyp t2)))
   | ExFlo loc s → mkexp loc (Pexp_constant (Const_float (uv s)))
   | ExFor loc i e1 e2 df el →
       let e3 = <:expr< do { $list:uv el$ } >> in
@@ -870,16 +874,16 @@ value rec expr =
   | ExFun loc pel →
       match uv pel with
       [ [(PaLab ploc lppo, w, e)] →
-          List.fold_right
-            (fun (p, po) e →
-               let lab = label_of_patt p in
-               let p =
-                 match uv po with
-                 [ Some p → p
-                 | None → p ]
-               in
-               mkexp loc (ocaml_pexp_function lab None [(patt p, e)]))
-           (uv lppo) (when_expr e (uv w))
+          match uv lppo with
+          [ [(p, po)] →
+              let lab = label_of_patt p in
+              let p =
+                match uv po with
+                [ Some p → p
+                | None → p ]
+              in
+              mkexp loc (ocaml_pexp_function lab None [mkpwe (p, w, e)])
+          | _ → error loc "bad AST" ]
       | [(PaNty loc s, w, e)] →
           match ocaml_pexp_newtype with
           [ Some newtype →
@@ -896,7 +900,7 @@ value rec expr =
           in
           mkexp loc
             (ocaml_pexp_function ("?" ^ lab) (option expr (uv eo))
-               [(patt p, when_expr e (uv w))])
+               [mkpwe (p, w, e)])
       | pel →
           let pel =
             if split_or_patterns_with_bindings then
@@ -972,7 +976,7 @@ value rec expr =
             match mto with
             [ Some mt →
                 let pt = package_of_module_type loc mt in
-                Pexp_constraint (mkexp loc e)
+                ocaml_pexp_constraint (mkexp loc e)
                   (Some (mktyp loc (ptyp_package pt))) None
             | None → e ]
           in
@@ -1036,10 +1040,12 @@ value rec expr =
            [("", expr e1); ("", expr e2)])
   | ExStr loc s →
       mkexp loc
-        (Pexp_constant (Const_string (string_of_string_token loc (uv s))))
+        (Pexp_constant
+           (ocaml_const_string (string_of_string_token loc (uv s))))
   | ExTry loc e pel → mkexp loc (Pexp_try (expr e) (List.map mkpwe (uv pel)))
   | ExTup loc el → mkexp loc (Pexp_tuple (List.map expr (uv el)))
-  | ExTyc loc e t → mkexp loc (Pexp_constraint (expr e) (Some (ctyp t)) None)
+  | ExTyc loc e t →
+      mkexp loc (ocaml_pexp_constraint (expr e) (Some (ctyp t)) None)
   | ExUid loc s →
       let ca = not Prtools.no_constructors_arity.val in
       let cloc = mkloc loc in
@@ -1091,7 +1097,7 @@ and mkjoinclause jc =
                   [ Some p → patt p
                   | None →
                       mkpat loc
-                        (ocaml_ppat_construct (Lident "()") (mkloc loc) None
+                        (ocaml_ppat_construct (mkloc loc) (Lident "()") None
                            False) ]
                 in
                 (mkloc locp, (mkloc loc, uv s), p))
@@ -1102,6 +1108,7 @@ and mkjoinclause jc =
   in
   (mkloc jc.jcLoc, jcval)
 and mkpe (p, e) =
+  let loc = Ploc.encl (loc_of_patt p) (loc_of_expr e) in
   let (p, e) =
     match e with
     [ ExTyc loc e (TyPol _ _ _ as t) → (PaTyc loc p t, e)
@@ -1113,7 +1120,7 @@ and mkpe (p, e) =
     [ PaTyc loc p (TyPot loc1 nt ct) → expand_gadt_type loc p loc1 nt ct e
     | p → (p, e) ]
   in
-  (patt p, expr e)
+  ocaml_value_binding (mkloc loc) (patt p) (expr e)
 and expand_gadt_type loc p loc1 nt ct e =
   let nt = uv nt in
   let e = <:expr< ($e$ : $ct$) >> in
@@ -1125,11 +1132,8 @@ and expand_gadt_type loc p loc1 nt ct e =
   let tp = List.map (fun s → "&" ^ s) nt in
   let ct = <:ctyp< ! $list:tp$ . $ct$ >> in
   (<:patt< ($p$ : $ct$) >>, e)
-and mkpwe (p, w, e) = (patt p, when_expr e (uv w))
-and when_expr e =
-  fun
-  [ Some w → mkexp (loc_of_expr e) (Pexp_when (expr w) (expr e))
-  | None → expr e ]
+and mkpwe (p, w, e) =
+  ocaml_case (patt p, option expr (uv w), mkloc (loc_of_expr e), expr e)
 and mklabexp (lab, e) =
   (patt_label_long_id lab, mkloc (loc_of_patt lab), expr e)
 and mkideexp (ide, e) = (ide, expr e)
@@ -1142,7 +1146,8 @@ and mktype_decl td =
          (ctyp t1, ctyp t2, mkloc loc))
       (uv td.tdCon)
   in
-  (uv (snd (uv td.tdNam)), type_decl (uv td.tdPrm) priv cl td.tdDef)
+  let tn = uv (snd (uv td.tdNam)) in
+  (tn, type_decl tn (uv td.tdPrm) priv cl td.tdDef)
 and module_type =
   fun
   [ MtAcc loc _ _ as f →
@@ -1179,15 +1184,20 @@ and sig_item s l =
   | SgDcl loc sl → List.fold_right sig_item (uv sl) l
   | SgDir loc _ _ → l
   | SgExc loc n tl →
-      [mksig loc (ocaml_psig_exception (uv n) (List.map ctyp (uv tl))) :: l]
+      [mksig loc
+         (ocaml_psig_exception (mkloc loc) (uv n) (List.map ctyp (uv tl))) ::
+       l]
   | SgExt loc n t p →
-      [mksig loc (ocaml_psig_value (uv n) (mkvalue_desc t (uv p))) :: l]
-  | SgInc loc mt → [mksig loc (Psig_include (module_type mt)) :: l]
+      let vn = uv n in
+      [mksig loc (ocaml_psig_value vn (mkvalue_desc vn t (uv p))) :: l]
+  | SgInc loc mt →
+      [mksig loc (ocaml_psig_include (mkloc loc) (module_type mt)) :: l]
   | SgMod loc rf ntl →
       if not (uv rf) then
         List.fold_right
           (fun (n, mt) l →
-             [mksig loc (ocaml_psig_module (uv n) (module_type mt)) :: l])
+             [mksig loc
+                (ocaml_psig_module (mkloc loc) (uv n) (module_type mt)) :: l])
           (uv ntl) l
       else
         match ocaml_psig_recmodule with
@@ -1198,21 +1208,25 @@ and sig_item s l =
             [mksig loc (psig_recmodule ntl) :: l]
         | None → error loc "no recursive module in this ocaml version" ]
   | SgMty loc n mt →
-      let si =
+      let mto =
         match mt with
-        [ MtQuo _ _ → Pmodtype_abstract
-        | _ → Pmodtype_manifest (module_type mt) ]
+        | MtQuo _ _ -> None
+        | _ -> Some (module_type mt)
+        end
       in
-      [mksig loc (ocaml_psig_modtype (uv n) si) :: l]
+      [mksig loc (ocaml_psig_modtype (mkloc loc) (uv n) mto) :: l]
   | SgOpn loc id →
-      [mksig loc (ocaml_psig_open (long_id_of_string_list loc (uv id))) :: l]
+      [mksig loc
+         (ocaml_psig_open (mkloc loc) (long_id_of_string_list loc (uv id))) ::
+       l]
   | SgTyp loc tdl →
       [mksig loc (ocaml_psig_type (List.map mktype_decl (uv tdl))) :: l]
   | SgUse loc fn sl →
       Ploc.call_with glob_fname (uv fn)
         (fun () → List.fold_right (fun (si, _) → sig_item si) (uv sl) l) ()
   | SgVal loc n t →
-      [mksig loc (ocaml_psig_value (uv n) (mkvalue_desc t [])) :: l]
+      let vn = uv n in
+      [mksig loc (ocaml_psig_value vn (mkvalue_desc vn t [])) :: l]
   | SgXtr loc _ _ → error loc "bad ast SgXtr" ]
 and module_expr =
   fun
@@ -1240,7 +1254,7 @@ and module_expr =
             [ Some mt →
                 let pt = package_of_module_type loc mt in
                 let t = mktyp loc (ptyp_package pt) in
-                mkexp loc (Pexp_constraint (expr e) (Some t) None)
+                mkexp loc (ocaml_pexp_constraint (expr e) (Some t) None)
             | None →
                 expr e ]
           in
@@ -1269,28 +1283,33 @@ and str_item s l =
   | StExc loc n tl sl →
       let si =
         match (uv tl, uv sl) with
-        [ (tl, []) → ocaml_pstr_exception (uv n) (List.map ctyp tl)
+        [ (tl, []) →
+            ocaml_pstr_exception (mkloc loc) (uv n) (List.map ctyp tl)
         | ([], sl) →
             match ocaml_pstr_exn_rebind with
             [ Some pstr_exn_rebind →
-                pstr_exn_rebind (uv n) (long_id_of_string_list loc sl)
+                pstr_exn_rebind (mkloc loc) (uv n)
+                  (long_id_of_string_list loc sl)
             | None →
                 error loc "no exception renaming in this ocaml version" ]
         | _ → error loc "renamed exception should not have parameters" ]
       in
       [mkstr loc si :: l]
-  | StExp loc e → [mkstr loc (Pstr_eval (expr e)) :: l]
+  | StExp loc e → [mkstr loc (ocaml_pstr_eval (expr e)) :: l]
   | StExt loc n t p →
-      [mkstr loc (ocaml_pstr_primitive (uv n) (mkvalue_desc t (uv p))) :: l]
+      let vn = uv n in
+      [mkstr loc (ocaml_pstr_primitive vn (mkvalue_desc vn t (uv p))) :: l]
   | StInc loc me →
       match ocaml_pstr_include with
-      [ Some pstr_include → [mkstr loc (pstr_include (module_expr me)) :: l]
+      [ Some pstr_include →
+          [mkstr loc (pstr_include (mkloc loc) (module_expr me)) :: l]
       | None → error loc "no include in this ocaml version" ]
   | StMod loc rf nel →
       if not (uv rf) then
         List.fold_right
           (fun (n, me) l →
-             [mkstr loc (ocaml_pstr_module (uv n) (module_expr me)) :: l])
+             let m = ocaml_pstr_module (mkloc loc) (uv n) (module_expr me) in
+             [mkstr loc m :: l])
           (uv nel) l
       else
         match ocaml_pstr_recmodule with
@@ -1311,9 +1330,12 @@ and str_item s l =
             [mkstr loc (pstr_recmodule nel) :: l]
         | None → error loc "no recursive module in this ocaml version" ]
   | StMty loc n mt →
-      [mkstr loc (ocaml_pstr_modtype (uv n) (module_type mt)) :: l]
+      let m = ocaml_pstr_modtype (mkloc loc) (uv n) (module_type mt) in
+      [mkstr loc m :: l]
   | StOpn loc id →
-      [mkstr loc (ocaml_pstr_open (long_id_of_string_list loc (uv id))) :: l]
+      [mkstr loc
+         (ocaml_pstr_open (mkloc loc) (long_id_of_string_list loc (uv id))) ::
+       l]
   | StTyp loc tdl →
       [mkstr loc (ocaml_pstr_type (List.map mktype_decl (uv tdl))) :: l]
   | StUse loc fn sl →
@@ -1375,7 +1397,8 @@ and class_sig_item c l =
       | None → error loc "no class constraint in this ocaml version" ]
   | CgDcl loc cl → List.fold_right class_sig_item (uv cl) l
   | CgInh loc ct →
-      [ocaml_class_type_field (mkloc loc) (Pctf_inher (class_type ct)) :: l]
+      [ocaml_class_type_field (mkloc loc)
+         (ocaml_pctf_inher (class_type ct)) :: l]
   | CgMth loc pf s t →
       [ocaml_class_type_field (mkloc loc)
          (ocaml_pctf_meth
@@ -1521,18 +1544,17 @@ value implem fname ast = do {
 
 value directive loc =
   fun
-  [ None → Pdir_none
-  | Some <:expr< $str:s$ >> → Pdir_string s
-  | Some <:expr< $int:i$ >> → Pdir_int (int_of_string_l loc i)
-  | Some <:expr< True >> →
+  [ <:expr< $str:s$ >> → Pdir_string s
+  | <:expr< $int:i$ >> → Pdir_int (int_of_string_l loc i)
+  | <:expr< True >> →
       match ocaml_pdir_bool with
       [ Some pdir_bool → pdir_bool True
       | None → error loc "no such kind of directive in this ocaml version" ]
-  | Some <:expr< False >> →
+  | <:expr< False >> →
       match ocaml_pdir_bool with
       [ Some pdir_bool → pdir_bool False
       | None → error loc "no such kind of directive in this ocaml version" ]
-  | Some e →
+  | e →
       let sl =
         loop e where rec loop =
           fun
@@ -1545,9 +1567,16 @@ value directive loc =
       Pdir_ident (long_id_of_string_list loc sl) ]
 ;
 
+value directive_args loc d =
+  match d with
+  | Some d -> directive loc d
+  | None -> Pdir_none
+  end
+;
+
 value phrase =
   fun
-  [ StDir loc d dp → Ptop_dir (uv d) (directive loc (uv dp))
+  [ StDir loc d dp → Ptop_dir (uv d) (directive_args loc (uv dp))
   | si → do {
       glob_fname.val := Plexing.input_file.val;
       Ptop_def (str_item si [])
