@@ -1,6 +1,6 @@
 (* camlp5r *)
-(* $Id: pa_macro.ml,v 6.12 2013-03-13 06:48:13 deraugla Exp $ *)
-(* Copyright (c) INRIA 2007-2012 *)
+(* pa_macro.ml,v *)
+(* Copyright (c) INRIA 2007-2014 *)
 
 #load "pa_extend.cmo";
 #load "q_MLast.cmo";
@@ -103,6 +103,7 @@ Added statements:
 
 open Pcaml;
 open Printf;
+open Versdep;
 
 type macro_value =
   [ MvExpr of list string and MLast.expr
@@ -125,11 +126,11 @@ value rec list_remove x =
 ;
 
 value oversion = do {
-  let v = String.copy Pconfig.ocaml_version in
+  let v = string_copy Pconfig.ocaml_version in
   for i = 0 to String.length v - 1 do {
     match v.[i] with
     [ '0'..'9' | 'a'..'z' | 'A'..'Z' -> ()
-    | _ -> v.[i] := '_' ];
+    | _ -> string_set v i '_' ];
   };
   v
 };
