@@ -1,6 +1,6 @@
 (* camlp5r *)
-(* $Id: q_ast.ml,v 6.27 2012-01-09 14:22:22 deraugla Exp $ *)
-(* Copyright (c) INRIA 2007-2012 *)
+(* q_ast.ml,v *)
+(* Copyright (c) INRIA 2007-2014 *)
 
 #load "pa_macro.cmo";
 #load "pa_extend.cmo";
@@ -215,6 +215,7 @@ module Meta_make (C : MetaSig) =
                lpoee]
       | ExIfe _ e1 e2 e3 → C.node "ExIfe" [expr e1; expr e2; expr e3]
       | ExInt _ s1 s2 → C.node "ExInt" [C.vala C.string s1; C.string s2]
+      | ExJdf _ lx e → C.node "ExJdf" [C.vala (C.list joinclause) lx; expr e]
       | ExLab _ lpoe →
           C.node "ExLab"
             [C.vala
@@ -246,13 +247,19 @@ module Meta_make (C : MetaSig) =
       | ExOvr _ lse →
           C.node "ExOvr"
             [C.vala (C.list (fun (s, e) → C.tuple [C.string s; expr e])) lse]
+      | ExPar _ e1 e2 → C.node "ExPar" [expr e1; expr e2]
       | ExPck _ me omt →
           C.node "ExPck" [module_expr me; C.option module_type omt]
       | ExRec _ lpe oe →
           C.node "ExRec"
             [C.vala (C.list (fun (p, e) → C.tuple [patt p; expr e])) lpe;
              C.option expr oe]
+      | ExRpl _ oe ls →
+          C.node "ExRpl"
+            [C.vala (C.option expr) oe;
+             (fun (_, s) → C.tuple [C.loc_v (); C.vala C.string s]) ls]
       | ExSeq _ le → C.node "ExSeq" [C.vala (C.list expr) le]
+      | ExSpw _ e → C.node "ExSpw" [expr e]
       | ExSnd _ e s → C.node "ExSnd" [expr e; C.vala C.string s]
       | ExSte _ e1 e2 → C.node "ExSte" [expr e1; expr e2]
       | ExStr _ s → C.node "ExStr" [C.vala C.string s]
@@ -352,6 +359,7 @@ module Meta_make (C : MetaSig) =
       | StClt _ lcict →
           C.node "StClt" [C.vala (C.list (class_infos class_type)) lcict]
       | StDcl _ lsi → C.node "StDcl" [C.vala (C.list str_item) lsi]
+      | StDef _ lx → C.node "StDef" [C.vala (C.list joinclause) lx]
       | StDir _ s oe →
           C.node "StDir" [C.vala C.string s; C.vala (C.option expr) oe]
       | StExc _ s lt ls →
@@ -384,6 +392,27 @@ module Meta_make (C : MetaSig) =
              C.vala (C.list (fun (p, e) → C.tuple [patt p; expr e])) lpe]
       | 
           StXtr loc s _ → C.xtr loc s ]
+    and joinclause x =
+      C.record
+        [(record_label "jcLoc", C.loc_v ());
+         (record_label "jcVal",
+          C.vala
+            (C.list
+               (fun (_, lllsop, e) →
+                  C.tuple
+                    [C.loc_v ();
+                     C.vala
+                       (C.list
+                          (fun (_, ls, op) →
+                             C.tuple
+                               [C.loc_v ();
+                                (fun (_, s) →
+                                   C.tuple [C.loc_v (); C.vala C.string s])
+                                  ls;
+                                C.vala (C.option patt) op]))
+                       lllsop;
+                     expr e]))
+            x.jcVal)]
     and type_decl x =
       C.record
         [(record_label "tdNam",
