@@ -5,14 +5,13 @@ INCL=
 FILE=
 OPTS="-mode T"
 PR_O=$1
-CAMLP5N=camlp5
+DEF=
 shift
 while test "" != "$1"; do
   case $1 in
   -I) INCL="$INCL -I $2"; shift;;
   -D*) OPTS="$OPTS $1";;
   -U*) OPTS="$OPTS $1";;
-  -name) CAMLP5N="$2"; shift;;
   *) FILE=$1;;
   esac
   shift
@@ -20,11 +19,10 @@ done
 
 set - `head -1 $FILE`
 if test "$2" = "camlp5r" -o "$2" = "camlp5"; then
-  if [ "$2" = "camlp5r" ]; then WHAT="${CAMLP5N}r"
-  else WHAT="${CAMLP5N}"; fi
+  if [ "$2" = "camlp5r" ]; then WHAT="${CAMLP5N}r"; else WHAT="${CAMLP5N}"; fi
   case "$WHAT" in
   ${CAMLP5N}r)
-    COMM="ocamlrun $DIR/../meta/$WHAT -nolib -I $DIR/../meta $INCL $DIR/../etc/$PR_O";;
+    COMM="${OCAMLN}run$EXE $DIR/../meta/$WHAT -nolib -I $DIR/../meta $INCL $DIR/../etc/$PR_O";;
   *) echo "not impl $WHAT" 1>&2; exit 2;;
   esac
   shift; shift
