@@ -1,6 +1,6 @@
 (* camlp5r pa_macro.cmo *)
-(* $Id: rprint.ml,v 6.17 2013-07-02 16:27:48 deraugla Exp $ *)
-(* Copyright (c) INRIA 2007-2012 *)
+(* rprint.ml,v *)
+(* Copyright (c) INRIA 2007-2014 *)
 
 IFDEF OCAML_VERSION >= OCAML_3_03 THEN
 
@@ -212,6 +212,9 @@ and print_simple_out_type ppf =
   | Otyp_manifest ty1 ty2 ->
       fprintf ppf "@[<2>%a ==@ %a@]" print_out_type ty1 print_out_type ty2
   | Otyp_abstract -> fprintf ppf "'abstract"
+  | IFDEF OCAML_VERSION >= OCAML_4_02_0 THEN
+    Otyp_open -> fprintf ppf "open"
+    END
   | Otyp_alias _ _ | Otyp_arrow _ _ _ | Otyp_constr _ [_ :: _] as ty ->
       fprintf ppf "@[<1>(%a)@]" print_out_type ty
   | IFDEF OCAML_VERSION >= OCAML_3_05 THEN
@@ -339,9 +342,15 @@ value rec print_out_class_type ppf =
               (print_typlist print_out_type ",") tyl ]
       in
       fprintf ppf "@[%a%a@]" pr_tyl tyl print_ident id
-  | Octy_fun lab ty cty ->
+  | IFDEF OCAML_VERSION < OCAML_4_02_0 THEN
+    Octy_fun lab ty cty ->
       fprintf ppf "@[%s[ %a ] ->@ %a@]" (if lab <> "" then lab ^ ":" else "")
         print_out_type ty print_out_class_type cty
+    ELSE
+    Octy_arrow lab ty cty ->
+      fprintf ppf "@[%s[ %a ] ->@ %a@]" (if lab <> "" then lab ^ ":" else "")
+        print_out_type ty print_out_class_type cty
+    END
   | Octy_signature self_ty csil ->
       let pr_param ppf =
         fun
@@ -379,9 +388,22 @@ value rec print_out_module_type ppf =
   | Omty_signature sg ->
       fprintf ppf "@[<hv 2>sig@ %a@;<1 -2>end@]" print_out_signature sg
   | Omty_functor name mty_arg mty_res ->
-      fprintf ppf "@[<2>functor@ (%s : %a) ->@ %a@]" name
-        print_out_module_type mty_arg print_out_module_type mty_res
-  | Omty_abstract -> () ]
+      IFDEF OCAML_VERSION < OCAML_4_02_0 THEN
+        fprintf ppf "@[<2>functor@ (%s : %a) ->@ %a@]" name
+          print_out_module_type mty_arg print_out_module_type mty_res
+      ELSE
+        match mty_arg with
+        [ Some mty_arg ->
+            fprintf ppf "@[<2>functor@ (%s : %a) ->@ %a@]" name
+              print_out_module_type mty_arg print_out_module_type mty_res
+        | None ->
+            fprintf ppf "@[<2>functor@ (%s) ->@ %a@]" name
+              print_out_module_type mty_res ]
+      END
+  | Omty_abstract -> ()
+  | IFDEF OCAML_VERSION >= OCAML_4_02_0 THEN
+    Omty_alias oi -> fprintf ppf "<rprint.ml: Omty_alias not impl>"
+    END ]
 and print_out_signature ppf =
   fun
   [ [] -> ()
@@ -391,8 +413,10 @@ and print_out_signature ppf =
         print_out_signature items ]
 and print_out_sig_item ppf =
   fun
-  [ Osig_exception id tyl ->
+  [ IFDEF OCAML_VERSION < OCAML_4_02_0 THEN
+    Osig_exception id tyl ->
       fprintf ppf "@[<2>exception %a@]" print_out_constr (id, tyl)
+    END
   | Osig_modtype name Omty_abstract ->
       fprintf ppf "@[<2>module type %s = 'a@]" name
   | Osig_modtype name mty ->
@@ -448,8 +472,11 @@ and print_out_type_decl kwd ppf x =
     IFDEF OCAML_VERSION <= OCAML_3_08_4 THEN
       let (name, args, ty, priv) = x in
       (name, args, ty, priv, [])
-    ELSE
+    ELSIFDEF OCAML_VERSION < OCAML_4_02_0 THEN
       x
+    ELSE
+      (x.otype_name, x.otype_params, x.otype_type, x.otype_private,
+       x.otype_cstrs)
     END
   in
   let constrain ppf (ty, ty') =
